@@ -570,6 +570,13 @@ class nftables(object):
                                                                     "op": "in",
                                                                     "right": "dnat"}},
                                                          {"accept": None}]}}})
+        default_rules.append({"add": {"rule":  {"family": "inet",
+                                                "table": TABLE_NAME,
+                                                "chain": "filter_%s" % "INPUT",
+                                                "expr": [{"match": {"left": {"meta": {"key": "iifname"}},
+                                                                    "op": "==",
+                                                                    "right": "lo"}},
+                                                         {"accept": None}]}}})
         if log_denied != "off":
             default_rules.append({"add": {"rule":  {"family": "inet",
                                                     "table": TABLE_NAME,
@@ -586,13 +593,6 @@ class nftables(object):
                                                                     "op": "in",
                                                                     "right": {"set": ["invalid"]}}},
                                                          {"drop": None}]}}})
-        default_rules.append({"add": {"rule":  {"family": "inet",
-                                                "table": TABLE_NAME,
-                                                "chain": "filter_%s" % "INPUT",
-                                                "expr": [{"match": {"left": {"meta": {"key": "iifname"}},
-                                                                    "op": "==",
-                                                                    "right": "lo"}},
-                                                         {"accept": None}]}}})
         for dispatch_suffix in ["POLICIES_pre", "ZONES", "POLICIES_post"]:
             default_rules.append({"add": {"chain": {"family": "inet",
                                                     "table": TABLE_NAME,
@@ -628,6 +628,13 @@ class nftables(object):
                                                                     "op": "in",
                                                                     "right": "dnat"}},
                                                          {"accept": None}]}}})
+        default_rules.append({"add": {"rule":  {"family": "inet",
+                                                "table": TABLE_NAME,
+                                                "chain": "filter_%s" % "FORWARD",
+                                                "expr": [{"match": {"left": {"meta": {"key": "iifname"}},
+                                                                    "op": "==",
+                                                                    "right": "lo"}},
+                                                         {"accept": None}]}}})
         if log_denied != "off":
             default_rules.append({"add": {"rule":  {"family": "inet",
                                                     "table": TABLE_NAME,
@@ -644,13 +651,6 @@ class nftables(object):
                                                                     "op": "in",
                                                                     "right": {"set": ["invalid"]}}},
                                                          {"drop": None}]}}})
-        default_rules.append({"add": {"rule":  {"family": "inet",
-                                                "table": TABLE_NAME,
-                                                "chain": "filter_%s" % "FORWARD",
-                                                "expr": [{"match": {"left": {"meta": {"key": "iifname"}},
-                                                                    "op": "==",
-                                                                    "right": "lo"}},
-                                                         {"accept": None}]}}})
         for dispatch_suffix in ["POLICIES_pre"]:
             default_rules.append({"add": {"chain": {"family": "inet",
                                                     "table": TABLE_NAME,
@@ -1252,10 +1252,6 @@ class nftables(object):
                                                               "field": "dport"}},
                                          "op": "==",
                                          "right": self._port_fragment(port)}})
-        if not rich_rule or type(rich_rule.action) != Rich_Mark:
-            expr_fragments.append({"match": {"left": {"ct": {"key": "state"}},
-                                             "op": "in",
-                                             "right": {"set": ["new", "untracked"]}}})
 
         rules = []
         if rich_rule:
@@ -1287,10 +1283,6 @@ class nftables(object):
         expr_fragments.append({"match": {"left": {"meta": {"key": "l4proto"}},
                                          "op": "==",
                                          "right": protocol}})
-        if not rich_rule or type(rich_rule.action) != Rich_Mark:
-            expr_fragments.append({"match": {"left": {"ct": {"key": "state"}},
-                                             "op": "in",
-                                             "right": {"set": ["new", "untracked"]}}})
 
         rules = []
         if rich_rule:
@@ -1307,6 +1299,7 @@ class nftables(object):
 
     def build_policy_tcp_mss_clamp_rules(self, enable, policy, tcp_mss_clamp_value,
                                       destination=None, rich_rule=None):
+        chain_suffix = "allow"
         table = "filter"
         _policy = self._fw.policy.policy_base_chain_name(policy, table, POLICY_CHAIN_PREFIX)
         add_del = { True: "add", False: "delete" }[enable]
@@ -1327,13 +1320,11 @@ class nftables(object):
         else:
             expr_fragments.append({"mangle": {"key": {"tcp option": {"name": "maxseg","field": "size"}},
                                               "value": tcp_mss_clamp_value}})
-        rules = []
-        rules.append({add_del: {"rule": {"family": "inet",
-                                "table": TABLE_NAME,
-                                "chain": "filter_%s_%s" % (_policy, chain_suffix),
-                                "expr": expr_fragments}}})
 
-        return rules
+        return [{add_del: {"rule": {"family": "inet",
+                           "table": TABLE_NAME,
+                           "chain": "filter_%s_%s" % (_policy, chain_suffix),
+                           "expr": expr_fragments}}}]
 
     def build_policy_source_ports_rules(self, enable, policy, proto, port,
                                       destination=None, rich_rule=None):
@@ -1354,10 +1345,6 @@ class nftables(object):
                                                               "field": "sport"}},
                                          "op": "==",
                                          "right": self._port_fragment(port)}})
-        if not rich_rule or type(rich_rule.action) != Rich_Mark:
-            expr_fragments.append({"match": {"left": {"ct": {"key": "state"}},
-                                             "op": "in",
-                                             "right": {"set": ["new", "untracked"]}}})
 
         rules = []
         if rich_rule:
@@ -1748,20 +1735,20 @@ class nftables(object):
         type_format = self._fw.ipset.get_ipset(name).type.split(":")[1].split(",")
 
         fragments = []
-        for i in range(len(type_format)):
-            if type_format[i] == "port":
+        for format in type_format:
+            if format == "port":
                 fragments.append({"meta": {"key": "l4proto"}})
                 fragments.append({"payload": {"protocol": "th",
                                               "field": "dport" if match_dest else "sport"}})
-            elif type_format[i] in ["ip", "net", "mac"]:
+            elif format in ["ip", "net", "mac"]:
                 fragments.append({"payload": {"protocol": self._set_get_family(name),
                                               "field": "daddr" if match_dest else "saddr"}})
-            elif type_format[i] == "iface":
+            elif format == "iface":
                 fragments.append({"meta": {"key": "iifname" if match_dest else "oifname"}})
-            elif type_format[i] == "mark":
+            elif format == "mark":
                 fragments.append({"meta": {"key": "mark"}})
             else:
-                raise FirewallError("Unsupported ipset type for match fragment: %s" % (type_format[i]))
+                raise FirewallError("Unsupported ipset type for match fragment: %s" % (format))
 
         return {"match": {"left": {"concat": fragments} if len(type_format) > 1 else fragments[0],
                           "op": "!=" if invert else "==",
@@ -1779,8 +1766,8 @@ class nftables(object):
             raise FirewallError(INVALID_ENTRY,
                                 "Number of values does not match ipset type.")
         fragment = []
-        for i in range(len(type_format)):
-            if type_format[i] == "port":
+        for i, format in enumerate(type_format):
+            if format == "port":
                 try:
                     index = entry_tokens[i].index(":")
                 except ValueError:
@@ -1798,7 +1785,7 @@ class nftables(object):
                 else:
                     fragment.append({"range": [port_str[:index], port_str[index+1:]]})
 
-            elif type_format[i] in ["ip", "net"]:
+            elif format in ["ip", "net"]:
                 try:
                     index = entry_tokens[i].index("/")
                 except ValueError:
